@@ -1,6 +1,6 @@
 package App::optex::pingu;
 
-my $VERSION = '1.01';
+my $VERSION = '1.02';
 
 use v5.24;
 use warnings;
@@ -15,6 +15,10 @@ use Data::Dumper;
 
 pingu - optex make-everything-pingu filter
 
+=head1 VERSION
+
+Version 1.02
+
 =head1 SYNOPSIS
 
 B<optex> -Mpingu [ options -- ] I<command>
@@ -227,8 +231,8 @@ C<pingu.asc4>:
 
 =back
 
-Coloring is done by L<Getopt::EX::Colormap> module.  See its document
-for detail.
+Coloring is done by L<Term::ANSIColor::Concise> module.  See its
+document for detail.
 
 =head1 INSTALL
 
@@ -307,7 +311,6 @@ as Perl itself.
 
 use File::Share qw(dist_dir);
 use List::Util qw(first pairmap);
-use Getopt::EX::Colormap qw(colorize);
 use Time::HiRes qw(usleep);
 use Scalar::Util;
 use Hash::Util qw(lock_keys);
